#version 330

layout(points) in;
layout(points) out;
layout(max_vertices = 20) out;

in float type0[];
in vec3 position0[];
in vec3 velocity0[];
in float age0[];

out float type1;
out vec3 position1;
out vec3 velocity1;
out float age1;

uniform float deltaTime;
uniform float time;
uniform float launcherLifetime;
uniform float shellLifetime;
uniform float secondaryShellLifetime;
uniform sampler1D randomTexture;

#define PARTICLE_TYPE_LAUNCHER 0.0f
#define PARTICLE_TYPE_SHELL 1.0f
#define PARTICLE_TYPE_SHELL2 2.0f

vec3 getRandomDirection(float coord) {
	vec3 direction = texture(randomTexture, coord).xyz;
	//direction -= vec3(0.5, 0.5, 0.5);
	return direction;
}

void main() {
	float age = age0[0] + deltaTime;
	// check for type
	if(type0[0] == PARTICLE_TYPE_LAUNCHER) {
		// particle is launcher
		if (age >= launcherLifetime) {
			for(int i = 0 ; i < 2 ; i++) {
				type1 = PARTICLE_TYPE_SHELL;
				position1 = position0[0];
				vec3 direction = getRandomDirection(time + i/2.0f);
				velocity1 = normalize(direction)*6.0f;
				age1 = 0.0;
				EmitVertex();
				EndPrimitive();
			}
			age = 0.0;
		}

		type1 = PARTICLE_TYPE_LAUNCHER;
		position1 = position0[0];
		velocity1 = velocity0[0];
		age1 = age;
		EmitVertex();
		EndPrimitive();
	} else {
		// particle is not launcher
		vec3 deltaP = deltaTime*velocity0[0];
		vec3 deltaV = deltaTime*vec3(0.0, 9.81, 0.0);

		if(type0[0] == PARTICLE_TYPE_SHELL) {
			// particle is first shell
			if(age < shellLifetime) {
				type1 = PARTICLE_TYPE_SHELL;
				position1 = position0[0] + deltaP;
	            velocity1 = velocity0[0] + deltaV;
	            age1 = age;

	            EmitVertex();
	            EndPrimitive();
	        } else {
                for(int i=0; i<3; i++) {
                     type1 = PARTICLE_TYPE_SHELL2;
                     position1 = position0[0];
                     vec3 direction = getRandomDirection((time + i/3.0f));
                     velocity1 = velocity0[0] + normalize(direction)*0.3f;
                     age1 = 0.0f;

                     EmitVertex();
                     EndPrimitive();
                }
            }
		} else {
			// particle is second shell
			if(age < secondaryShellLifetime) {
                type1 = PARTICLE_TYPE_SHELL2;
                position1 = position0[0] + deltaP;
                velocity1 = velocity0[0] + deltaV;
                age1 = age;

                EmitVertex();
                EndPrimitive(); 
            }
		}
	}
}
