#version 330 core

uniform mat4 M;
uniform sampler2D textureSampler;
uniform sampler2DShadow shadowMap;
uniform vec3 cameraPosition;
uniform vec3 lightPosition;
uniform vec3 gamma;

in vec3 fragmentVertex;
in vec2 fragmentTexCoord;
in vec3 fragmentNormal;
in vec4 shadowCoord;

out vec4 finalColor;

const vec2 poisson[16] = vec2[](
   vec2(-0.94201624, -0.39906216), 
   vec2(0.94558609, -0.76890725), 
   vec2(-0.094184101, -0.92938870), 
   vec2(0.34495938, 0.29387760), 
   vec2(-0.91588581, 0.45771432), 
   vec2(-0.81544232, -0.87912464), 
   vec2(-0.38277543, 0.27676845), 
   vec2(0.97484398, 0.75648379), 
   vec2(0.44323325, -0.97511554), 
   vec2(0.53742981, -0.47373420), 
   vec2(-0.26496911, -0.41893023), 
   vec2(0.79197514, 0.19090188), 
   vec2(-0.24188840, 0.99706507), 
   vec2(-0.81409955, 0.91437590), 
   vec2(0.19984126, 0.78641367), 
   vec2(0.14383161, -0.14100790) 
);

float rand(vec3 number, int i) {
	vec4 vector = vec4(number, i);
	float product = dot(vector, vec4(12.9898,78.233,45.164,94.673));
	return fract(sin(product)*43758.5453);
}

// currently one white light is used (represented by vec3(1,1,1)
void main() {
	vec3 normal = fragmentNormal;
	vec3 surfacePosition = fragmentVertex;
	vec4 surfaceColor = texture(textureSampler, fragmentTexCoord);
	vec3 surfaceToLight = normalize(lightPosition - surfacePosition);
	vec3 surfaceToCamera = normalize(cameraPosition - surfacePosition);

	// ambient part (ambient coeff. * color * intensity)
	float ambientCoeff = 0.01;
	vec3 ambientPart = ambientCoeff*surfaceColor.rgb*vec3(1,1,1);
	// diffuse part (diffuse coeff. * color * intensity)
	float NL = dot(normal, surfaceToLight);
	float diffuseCoeff = max(0.0, NL);
	vec3 diffusePart = diffuseCoeff*surfaceColor.rgb*vec3(1,1,1);
	// specular part
	float specularCoeff = 0.0;
	if(diffuseCoeff > 0.0)
		specularCoeff = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, normal))), 80.0); 
	vec3 specularPart = specularCoeff*vec3(1,1,1)*vec3(1,1,1);

	// shadow sampling
	float bias = 0.003;
	//float bias = 0.0005*tan(acos(NL));
	//bias = clamp(bias, 0, 0.002);

	float visibility = 1.0;
	for(int i=0; i<4; i++) {
		//int index = int(16*rand(floor(surfacePosition.xyz*1000.0), i))%16;
		//int index = i;
		int index = int(mod(16.0*rand(gl_FragCoord.xyy, i), 16));
		visibility -= 0.2*(1.0-texture(shadowMap, vec3(shadowCoord.xy + poisson[index]/1000.0, (shadowCoord.z-bias)/shadowCoord.w)));
	}

	// attenuation
	float distanceToLight = length(lightPosition - surfacePosition);
	float attenuation = 1.0/(1.0 + 0.2*pow(distanceToLight, 2));
	// color before gamma correction
	vec3 nogammaColor = ambientPart + attenuation*visibility*(diffusePart*2500 + specularPart*500);
	// final color
	finalColor = vec4(pow(nogammaColor, gamma), surfaceColor.a);
}
