#version 330 core

uniform mat4 MVP;
uniform mat4 M;
uniform mat4 biasDepthVP;
uniform mat4 bones[100];

layout(location = 0) in vec3 vertex;
layout(location = 1) in vec2 vertexTexCoord;
layout(location = 2) in vec3 vertexNormal;
layout(location = 3) in ivec4 boneIDs;
layout(location = 4) in vec4 weights;

out vec3 fragmentVertex;
out vec2 fragmentTexCoord;
out vec3 fragmentNormal;
out vec4 shadowCoord;

void main() {
	// bone calculations
	mat4 transform = bones[boneIDs[0]]*weights[0];
	transform += bones[boneIDs[1]]*weights[1];
	transform += bones[boneIDs[2]]*weights[2];
	transform += bones[boneIDs[3]]*weights[3];

	vec4 transformedPosition = transform*vec4(vertex, 1.0);
	gl_Position = MVP*transformedPosition;

	// handling normals
	vec4 transformedNormal = transform*vec4(vertexNormal, 0.0);
	fragmentNormal = (M*transformedNormal).xyz;

	// handling world position
	fragmentVertex = (M*transformedPosition).xyz;

	fragmentTexCoord = vertexTexCoord;
	shadowCoord = biasDepthVP*M*vec4(vertex, 1);
}
